//============================================================================
// Name        : download.cpp
// Author      : Jorge Campos de la Mata
// Version     : 1.0
// Copyright   :
// Description : Download a given file from the storage server
//============================================================================

#include <cstdlib>
#include <fstream>
#include <iostream>
#include <vector>
#include <stdio.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <MLE.h>
#include <aux.h>
#include <json/json.h>

using namespace std;

string RETRIEVE_FILE_ENDPOINT = "http://localhost/~jorge_cmata/retrieve_file.php";


void convDecryption(string keyfilename, string tagsfilename){
	ifstream keyfile(keyfilename, ifstream::binary);
	string filename;
	string key;
	/************ GET KEY ************/
	if (keyfile.is_open())
		{
		  string line;
		  //Read the file line by line
		  while (keyfile >> line)
		  {
			  key = hexDecode(line); //First line is the hexadecimal personal Key
		  }
		  keyfile.close();
	}else{
		  cout << "Unable to open key file";
	}

	/************ GET TAGS VECTOR AND CREATE JSON ************/
	ifstream tagsfile(tagsfilename, ifstream::binary);
	Json::Value root;
	vector<string> taglist;
	if (tagsfile.is_open())
		{
		  string line;
		  int position = 0;
		  //Read the file line by line
		  while (tagsfile >> line)
		  {
			  if(position==0){
				  filename = line; //Filename in fist line
			  } else {
				  taglist.push_back(line); //Set tag in the last position
				  root[position-1]["tag"] = line; //Filling JSON root
			  }
			  position++;
		  }
		  tagsfile.close();
	}else{
		  cout << "Unable to open tags file";
	}

	  Json::FastWriter fastWriter;
	  string json = fastWriter.write(root);

	  /************ SEND REQUEST ************/
	  string result = cURLpostAndGetResponse(json, RETRIEVE_FILE_ENDPOINT);

	  /************ PARSE JSON RESPONSE AND GET ENCRYPTED TEXT ************/

	  string originalContent;
	  Json::Value rootRes;
	  Json::Reader reader;
	  cout << result.c_str() << endl;
	  bool parsingSuccessful = reader.parse( result.c_str(), rootRes );
	  if ( !parsingSuccessful ){ //ERROR
		  cout  << "Failed to parse" << reader.getFormattedErrorMessages();
	  } else { //SUCCESS
		  for (unsigned n=0; n<taglist.size(); ++n) {
			  //Append content to the result
			  Json::FastWriter fastWriterRes;
			  //Decryption
			  string ciphertextHex = fastWriterRes.write(rootRes[n]["content"]);
			  ciphertextHex.erase(remove(ciphertextHex.begin(), ciphertextHex.end(), '"'), ciphertextHex.end()); //Delete quotes in the response
			  string ciphertext = hexDecode(ciphertextHex);
			  string decrypted = decrypt(key, ciphertext);
			  if(n==(taglist.size()-1)){ //Last chunk
				  decrypted = removePadding(decrypted);
			  }
			  originalContent.append(decrypted);
		  }
	  }

	  /******************** ORIGINAL FILE CREATION *************************/

	  ofstream originalfile;
	  cout << "filename " << filename << endl;
	  originalfile.open (filename); //Overwrite
	  if (originalfile.is_open())
	  {
		  originalfile << originalContent;
		  originalfile.close();
		  cout << "File successfully downloaded";
	  }
	  else cerr << "Unable to open file";
}

void mleDecryption(string keyfilename){
	ifstream keyfile(keyfilename, ifstream::binary);
	string filename;
	if (keyfile.is_open())
	    {
		  /************ GET KEYS ANDS TAGS ************/
		  string line;
		  vector<string> keylist;
		  vector<string> taglist;
		  bool isFirstPosition = true;
		  //Read the file line by line
	      while (keyfile >> line)
	      {
	    	  if(isFirstPosition){ //No keys in first position --> Only filename
	    		  filename = line;
	    		  isFirstPosition = false;
	    	  } else {
	    		  string key = hexDecode(line); //Each line is a Hexadecimal Key
				  keylist.push_back(key); //Set in the last position
				  taglist.push_back(tagGen(key)); //Set in the last position
	    	  }
	      }
	      keyfile.close();

	      /************ CREATE A JSON WITH THE TAGS ************/
		  Json::Value root;
		  for (unsigned n=0; n<taglist.size(); ++n) {
		  	  //Add the tag to the JSON object
		  	  root[n]["tag"] = taglist.at(n);
		  }

		  Json::FastWriter fastWriter;
		  string json = fastWriter.write(root);

		  /************ SEND REQUEST ************/
		  string result = cURLpostAndGetResponse(json, RETRIEVE_FILE_ENDPOINT);

		  /************ PARSE JSON RESPONSE AND GET ENCRYPTED TEXT ************/

		  string originalContent;
		  Json::Value rootRes;
		  Json::Reader reader;
		  bool parsingSuccessful = reader.parse( result.c_str(), rootRes );
		  if ( !parsingSuccessful ){ //ERROR
			  cout  << "Failed to parse" << reader.getFormattedErrorMessages();
		  } else { //SUCCESS
			  for (unsigned n=0; n<keylist.size(); ++n) {
			  	  //Append content to the result
				  Json::FastWriter fastWriterRes;
				  //Decryption
				  string ciphertextHex = fastWriterRes.write(rootRes[n]["content"]);
				  ciphertextHex.erase(remove(ciphertextHex.begin(), ciphertextHex.end(), '"'), ciphertextHex.end()); //Delete quotes in the response
				  string ciphertext = hexDecode(ciphertextHex);
				  string decrypted = decrypt(keylist[n], ciphertext);
				  if(n==(keylist.size()-1)){ //Last chunk
					  decrypted = removePadding(decrypted);
				  }
			  	  originalContent.append(decrypted);
			  }
		  }

		  /******************** ORIGINAL FILE CREATION *************************/

		  ofstream originalfile;
		  cout << "filename " << filename << endl;
		  originalfile.open (filename); //Overwrite
		  if (originalfile.is_open())
		  {
			  originalfile << originalContent;
			  originalfile.close();
			  cout << "File successfully downloaded";
		  }
		  else cerr << "Unable to open file";
	    }
	  else{
		  cout << "Unable to open file";
	  }
}

int main1(int argc, char* argv[])
{

	/*********************** ARGUMENTS MANAGEMENT *******************************/

  /* ERROR CASE: Different number of arguments than expected. Help message */
  if (argc < 2 || argc > 4)
    {
      cerr << "------------------HELP MESSAGE------------------" << endl;
      cerr << " Incorrect number of arguments. (1) decryption mode (2) key file  (3) tags file" << endl;
      cerr << "decryption mode -> 'mle' to Message-Locked Encryption or 'conv' to conventional decryption" << endl;
      cerr << "key file -> file with the key" << endl;
      cerr << "tags file (only required in 'conv' decryption mode) -> file with the tags of the file which is desired to download" << endl;
      return 1;
    }

  string decryption_mode = argv[1];
  if(decryption_mode == "conv"){
	  convDecryption(argv[2], argv[3]);
  } else if(decryption_mode == "mle"){
	  mleDecryption(argv[2]);
  } else{
	  cerr << "Wrong decryption mode." << endl;
	  return 1;
  }

}
