/*
 * aux.cpp
 *
 *  Created on: 7 de feb. de 2017
 *      Author: jorge_cmata
 *
 * These functions are created to reduce the effort in task related with encode/decode in base64
 */

#include <cryptopp/base64.h>
#include <cryptopp/hex.h>
#include <stdint.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include <curl/curl.h>


using namespace std;


/**
 * Auxiliary function for management of responses in cURL
 */
size_t CurlWrite_CallbackFunc_StdString(void *contents, size_t size, size_t nmemb, std::string *s)
{
    size_t newLength = size*nmemb;
    size_t oldLength = s->size();
    try
    {
        s->resize(oldLength + newLength);
    }
    catch(std::bad_alloc &e)
    {
        //handle memory problem
        return 0;
    }

    std::copy((char*)contents,(char*)contents+newLength,s->begin()+oldLength);
    return size*nmemb;
}

/**
 * Encode a string passed by parameter in base64
 */
string base64Encode(string input){
	string encoded;

	CryptoPP::StringSource ss((byte *)input.c_str(), sizeof((byte *)input.c_str()), true,
		    new CryptoPP::Base64Encoder(
		        new CryptoPP::StringSink(encoded)
		    ) // Base64Encoder
		); // StringSource

	return encoded;
}

/**
 * Send a post request over HTTP to a given endpoint.
 * Params: fileContent and endpoint
 */

void cURLpost(string json, string endpoint){

	  CURL *curl;
	  CURLcode res;
	  struct curl_slist *headers = NULL;
	  headers = curl_slist_append(headers, "Accept: application/json");
	  headers = curl_slist_append(headers, "Content-Type: application/json");
	  headers = curl_slist_append(headers, "charsets: utf-8");

	  curl = curl_easy_init();
	  if(curl) {
		  curl_easy_setopt(curl, CURLOPT_URL, endpoint.c_str());
		  curl_easy_setopt(curl, CURLOPT_HTTPHEADER, headers);
		  curl_easy_setopt(curl, CURLOPT_POSTFIELDS, (const char*)json.c_str());
		  //curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L); // enable verbose for easier tracing

		  res = curl_easy_perform(curl);
		  /* Check for errors */
		  if(res != CURLE_OK) {
			  cerr << "curl_easy_perform() failed: " << res << endl;
		  }
		  else {
			cout << "File successfully sent" << endl;
		  }
		  curl_easy_cleanup(curl); //cleanup
		}
}

/**
 * Send a post request over HTTP to a given endpoint and listen for the response
 * Params: fileContent and endpoint
 */
string cURLpostAndGetResponse(string json, string endpoint){

	  CURL *curl;
	  CURLcode res;
	  std::string s;
	  struct curl_slist *headers = NULL;
	  headers = curl_slist_append(headers, "Accept: application/json");
	  headers = curl_slist_append(headers, "Content-Type: application/json");
	  headers = curl_slist_append(headers, "charsets: utf-8");

	  curl = curl_easy_init();
	  if(curl) {
		  curl_easy_setopt(curl, CURLOPT_URL, endpoint.c_str());
		  curl_easy_setopt(curl, CURLOPT_HTTPHEADER, headers);
		  curl_easy_setopt(curl, CURLOPT_POSTFIELDS, (const char*)json.c_str());
		  //curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L); // enable verbose for easier tracing
		  curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, CurlWrite_CallbackFunc_StdString);
		  curl_easy_setopt(curl, CURLOPT_WRITEDATA, &s);

		  res = curl_easy_perform(curl);
		  /* Check for errors */
		  if(res != CURLE_OK) {
			  cerr << "curl_easy_perform() failed: " << res << endl;
		  }
		  else {
			  cout << "File successfully sent" << endl;
		  }
		  curl_easy_cleanup(curl); //cleanup
		  return s;
		}
}




