//============================================================================
// Name        : upload.cpp
// Author      : Jorge Campos de la Mata
// Version     : 1.0
// Copyright   :
// Description : Upload a given file to a storage server
//============================================================================

#include <cstdlib>
#include <fstream>
#include <iostream>
#include <vector>
#include <stdio.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>

#include <MLE.h>
#include <aux.h>
#include <json/json.h>


using namespace std;


string STORE_FILE_ENDPOINT = "http://localhost/~jorge_cmata/store_file.php";
string chunking_mode;
string keygen_mode;
string personal_key = "";

/**
 * Prints the help usage message
 */
void printHelpMessage()
{
	cerr << "------------------HELP MESSAGE------------------" << endl;
	cerr << "usage: (1) input_file (2) keygen_mode (3) chunk-algorithm (4)[chunk_size] (4)[min_chunk_size] (5)[max_chunk_size] (6)[window_size]" << endl;
	cerr << "input_file -> name of the file" << endl;
	cerr << "keygen_mode -> 'mle' to MLE encryption" << endl;
	cerr << "               'conv' to conventional encryption (the personal key will be generated)" << endl;
	cerr << "               'key_file' to conventional encryption with the input key" << endl;
	cerr << "chunk-algorithm -> 'fixed' to fixed-block size algorithm " << endl;
	cerr << "                   'cdc' to basic-sliding window " << endl;
	cerr << "                   'tttd' to 2-threshold 2 divisors " << endl;
	cerr << "                   'lb' to leap-based CDC algorithm " << endl;
	cerr << "chunk-size (optional) -> Only for fixed-block algorithm. It represents the chunk size (in KB). Default value is set in 8KB" << endl;
	cerr << "minimun-chunk-size (optional) -> Only for CDC algorithm. It represents the minimum chunk size (in KB). Default value is set in 4KB" << endl;
	cerr << "maximum-chunk-size (optional) -> Only for CDC algorithm. It represents the maximum chunk size (in KB). Default value is set in 12KB" << endl;
	cerr << "window-size (optional) -> Only for CDC algorithm. It represents the window size (in B). Default value is set in 64B" << endl;
}

void MLE(Json::Value &root, string plaintext, string filename, int position){

	  size_t lastindex = filename.find_last_of(".");
	  string rawFilename = filename.substr(0, lastindex);

	  //We get the key encoded from bytes to hexadecimal (readable)
	  string keyHex = keyGen(plaintext);

	  //Save the keys in a file
	  ofstream keyStoragefile;
	  string storageKeyFilename = "/Users/jorge_cmata/Documents/workspace/cloudclient/traces/" + rawFilename + "_storagekey.txt";
	  //Check if the file already exists (only in the first iteration)
	  if(position==0 && access( storageKeyFilename.c_str(), F_OK ) != -1) {
		  // Delete the old file if it already exists
		  if( remove( storageKeyFilename.c_str() ) != 0 )
			  cerr << "Error deleting the file " << storageKeyFilename << endl;
		  else
			  cout << "File " << storageKeyFilename << " successfully deleted" << endl;
	  }
	  //Create the file
	  keyStoragefile.open (storageKeyFilename, ios::app); //Append
	  if (keyStoragefile.is_open())
		{
		  if(position==0){ //In the first loop, we add the filename
			  keyStoragefile << filename << endl;
		  }
		  keyStoragefile << keyHex << endl;
		  keyStoragefile.close();
		}
	  else cerr << "Unable to open file";

	  //Decode the key from hexadecimal to bytes
	  string key = hexDecode(keyHex);

	  //Get the ciphertext
	  string ciphertext = encrypt(key, plaintext);
	  string ciphertextHex = hexEncode(ciphertext);

	  //Generate a Tag in hexadecimal (readable)
	  string tagHex = tagGen(key);

	  //Save tag and ciphertext in json

	  root[position]["content"] = ciphertextHex;
	  root[position]["tag"] = tagHex;
}

void convEncrypt(Json::Value &root, string plaintext, string filename, int position){

	  size_t lastindex = filename.find_last_of(".");
	  size_t firstIndex = filename.find_last_of("/");
	  string rawFilename = filename.substr(firstIndex, lastindex);

	  //Check if the user has already a personal key or not
	  if(personal_key=="")//generate a new one
	  {
		  personal_key = generateRandomKey();
		  //Save the keys in a file
		  ofstream keyStoragefile;
		  string storageKeyFilename = "/Users/jorge_cmata/Documents/workspace/cloudclient/traces/storage_key.txt";
		  //Check if the file already exists (only in the first iteration)
		  if(position==0 && access( storageKeyFilename.c_str(), F_OK ) != -1) {
			// Delete the old file if it already exists
			if( remove( storageKeyFilename.c_str() ) != 0 )
				 cerr << "Error deleting the file " << storageKeyFilename << endl;
			else
				cout << "File " << storageKeyFilename << " successfully deleted" << endl;
		  }
		  //Create the file
		  keyStoragefile.open (storageKeyFilename, ios::app); //Append
		  if (keyStoragefile.is_open())
		   {
				 keyStoragefile << personal_key << endl;
				 keyStoragefile.close();
			}
			else cerr << "Unable to open file";
	  }

	  //Decode the key from hexadecimal to bytes
	  string key = hexDecode(personal_key);

	  //Get the ciphertext
	  string ciphertext = encrypt(key, plaintext);
	  string ciphertextHex = hexEncode(ciphertext);

	  //Generate a Tag in hexadecimal (readable)
	  string tagHex = tagGen(ciphertext);

	  //Create or refresh tags file
	  ofstream tagsStoragefile;
	  string tagsStoragefilename = "/Users/jorge_cmata/Documents/workspace/cloudclient/traces/" + rawFilename + "_storagetags.txt";
	  //Check if the file already exists (only in the first iteration)
	  if(position==0 && access( tagsStoragefilename.c_str(), F_OK ) != -1) {
		  // Delete the old file if it already exists
		  if( remove( tagsStoragefilename.c_str() ) != 0 )
			  cerr << "Error deleting the file " << tagsStoragefilename << endl;
		  else
			  cout << "File " << tagsStoragefilename << " successfully deleted" << endl;
	  }
	  tagsStoragefile.open(tagsStoragefilename, ios::app); //Append
	  if (tagsStoragefile.is_open())
		{
		  if(position==0){ //In the first loop, we add the filename
			tagsStoragefile << filename << endl;
		  }
		tagsStoragefile << tagHex << endl;
		tagsStoragefile.close();
		}
	  else cerr << "Unable to open file";

	  //Save tag and ciphertext in json

	  root[position]["content"] = ciphertextHex;
	  root[position]["tag"] = tagHex;
}

/**
 * FIXED BLOCK SIZE DEDUPLICATION
 */
void fixedBlocksMode(string filename, size_t total_size, size_t chunk_size){

	ifstream file(filename, ifstream::binary);

	//Starting point to meauser the execution time
	clock_t tStart = clock();

	/****************************** CHUNKING **************************************/

	  Json::Value root;
	  ofstream originalfile;

	  size_t total_chunks = total_size / chunk_size;
	  size_t last_chunk_size = total_size % chunk_size;

	  if (last_chunk_size != 0) /* if the above division was uneven */
	    {
		  //add an unfilled final chunk
	      ++total_chunks;
	    }
	  else /* if division was even, last chunk is full */
	    {
	      last_chunk_size = chunk_size;
	    }

	  /* the loop of chunking */
	  for (int chunk = 0; chunk < total_chunks; ++chunk)
	    {
		  size_t this_chunk_size;
		  /* if last chunk */
		  if(chunk == total_chunks - 1) /* then fill chunk with remaining bytes */
		  {
			  this_chunk_size = last_chunk_size;
		  } else{ /* else full entire chunk */
			  this_chunk_size = chunk_size;
		  }

		  /* adapt this portion as necessary, this is the fast C++ way */
		  vector<char> chunk_data(this_chunk_size);
		  file.read(&chunk_data[0], /* address of buffer start */
					this_chunk_size); /* this many bytes is to be read */

		  string plaintext = string(chunk_data.begin(), chunk_data.end());

		  //Fill the last chunk with 1+0s to get the same size in all the chunks
		  if(chunk == total_chunks - 1)
		  {
			 /** plaintext.append(1, '1');
			  size_t number_of_zeros = chunk_size - last_chunk_size - 1;
			  plaintext.append(number_of_zeros, '0');**/ //Uncomment these lines to enable padding
		  }

	     // cout << "chunk# " << chunk << " with size : " << plaintext.size() << endl;


		  if(keygen_mode == "conv"){
			  // CONVENTIONAL ENCRYPTION
			   convEncrypt(root, plaintext, filename, chunk);
		  } else{
			  // MESSAGE LOCKED ENCRYPTION
			   MLE(root, plaintext, filename, chunk);
		  }

	    }

	  /*************************** SEND TAGS AND CIPHERTEXT ****************************/

	  double time_taken = (double)(clock() - tStart)/CLOCKS_PER_SEC;

	  Json::FastWriter fastWriter;
	  string json = fastWriter.write(root);
	  cURLpost(json, STORE_FILE_ENDPOINT);

	  double time_taken_1 = (double)(clock() - tStart)/CLOCKS_PER_SEC;

	  //FIXED BLOCK ALGORITHM ANAlYSIS LOGS
	  cout << "Time taken: " << time_taken << " ||" << endl;
	  cout << "Time taken 1: " << time_taken_1 << " ||" << endl;
	  cout << "Number of chunks: " << total_chunks << " ||" << endl;
	  double chunk_size_average = (chunk_size*(total_chunks-1) + last_chunk_size)/total_chunks;
	  cout << "Chunk size average: " << chunk_size_average << " ||" << endl;
	  cout << "last chunk size: " << last_chunk_size << " ||" << endl;
}


void bswMode(string filename, size_t total_size, size_t min_chunk_size, size_t max_chunk_size, size_t window_size){

	Json::Value root;

	ifstream file(filename, ifstream::binary);
	vector<char> data(total_size);
	file.read(&data[0], total_size); //Get the file content

	//Starting point to meauser the execution time
	clock_t tStart = clock();

	string filedata = string(data.begin(),data.end());

	bool fileFinished = false;
	int startpoint_chunk = 0;
	size_t new_max_chunk_size = max_chunk_size;
	int number_of_chunks = 0;
	int number_of_times = 0;
	int number_of_forced_blocks = 0;

	while(!fileFinished){
		string chunk;
		//Detect if the chunk is the last one and its size is less than minimum chunk size
		if(startpoint_chunk + min_chunk_size >total_size){
			fileFinished = true;
			//Add padding to complete the chunk
			chunk = filedata.substr(startpoint_chunk, total_size - startpoint_chunk);
			//Fill the last chunk with 1+0s to get the same size in all the chunks
			/**chunk.append(1, '1');
			size_t number_of_zeros = min_chunk_size - (total_size -startpoint_chunk) - 1;
			chunk.append(number_of_zeros, '0');**/ //Uncomment these lines to enable padding
		}
		else{
			//Detect if the chunk is the last one and its size is between minimum and maximum (we have to set a new maximum threshold)
			if(startpoint_chunk + max_chunk_size > total_size){
				fileFinished = true;
				new_max_chunk_size = total_size - startpoint_chunk;
			}

			//-----------BSW ALGORITHM-----------
			int buffer = 0;
			string window;
			int startpoint_window = min_chunk_size - window_size;
			bool isBoundaryFound = false;
			while(startpoint_window + window_size < new_max_chunk_size){ //Is between min and max

				window = filedata.substr(startpoint_chunk + startpoint_window, window_size);
				number_of_times++;
				if(checkRollingHash(window)){ //if window content satisfies the condition
					//New boundary found
					fileFinished = false;
					chunk = filedata.substr(startpoint_chunk, min_chunk_size + buffer);
					isBoundaryFound = true;
					break;
				} else{
					//Move the sliding window 1 byte
					startpoint_window++;
					buffer++;
				}
			} //Maximum chunk size reached


			//If the boundary has not been found, force a chunk with the maximum chunk size
			if(!isBoundaryFound){
				number_of_forced_blocks++;
				chunk = filedata.substr(startpoint_chunk, new_max_chunk_size);
			}
			//update variables for the next loop
			isBoundaryFound = false;
			buffer = 0;
			startpoint_chunk = startpoint_chunk + chunk.size();
		}

		number_of_chunks++;

		if(keygen_mode == "conv"){
			// CONVENTIONAL ENCRYPTION
			convEncrypt(root, chunk, filename, number_of_chunks-1); //position = number_of_chunks-1
		} else{
			// MESSAGE LOCKED ENCRYPTION
			MLE(root, chunk, filename, number_of_chunks-1);//position = number_of_chunks-1
		}
	}



	/*************************** SEND TAGS AND CIPHERTEXT ****************************/

	double time_taken = (double)(clock() - tStart)/CLOCKS_PER_SEC;

	Json::FastWriter fastWriter;
	string json = fastWriter.write(root);
	cURLpost(json, STORE_FILE_ENDPOINT);

	//CDC ALGORITHM ANAlYSIS LOGS
	cout << "Time taken: " << time_taken << " ||" << endl;
	cout << "Number of chunks: " << number_of_chunks << " ||" << endl;
	cout << "Chunk size average: " << total_size/number_of_chunks << " ||" << endl;
	cout << "Number of forced chunks " << number_of_forced_blocks << " ||" << endl;
	cout << "Number of times hash function " << number_of_times << " ||" << endl;

}

void tttdMode(string filename, size_t total_size, size_t min_chunk_size, size_t max_chunk_size, size_t window_size){

	Json::Value root;

	ifstream file(filename, ifstream::binary);
	vector<char> data(total_size);
	file.read(&data[0], total_size); //Get the file content

	//Starting point to meauser the execution time
	clock_t tStart = clock();

	string filedata = string(data.begin(),data.end());

	bool fileFinished = false;
	int startpoint_chunk = 0;
	size_t new_max_chunk_size = max_chunk_size;
	int number_of_chunks = 0;
	int number_of_times = 0;
	int number_of_forced_blocks = 0;
	int number_of_2d_blocks = 0;

	while(!fileFinished){
		string chunk;
		//Detect if the chunk is the last one and its size is less than minimum chunk size
		if(startpoint_chunk + min_chunk_size >total_size){
			fileFinished = true;
			//Add padding to complete the chunk
			chunk = filedata.substr(startpoint_chunk, total_size - startpoint_chunk);
			//Fill the last chunk with 1+0s to get the same size in all the chunks
			/**chunk.append(1, '1');
			size_t number_of_zeros = min_chunk_size - (total_size -startpoint_chunk) - 1;
			chunk.append(number_of_zeros, '0');**/ //Uncomment these lines to enable padding
		}
		else{
			//Detect if the chunk is the last one and its size is between minimum and maximum (we have to set a new maximum threshold)
			if(startpoint_chunk + max_chunk_size > total_size){
				fileFinished = true;
				new_max_chunk_size = total_size - startpoint_chunk;
			}

			//-----------TTTD ALGORITHM-----------
			int buffer = 0;
			string window;
			int startpoint_window = min_chunk_size - window_size;
			bool isBoundaryFound = false;
			int possibleBoundary = 0;
			while(startpoint_window + window_size < new_max_chunk_size){ //Is between min and max

				window = filedata.substr(startpoint_chunk + startpoint_window, window_size);
				number_of_times++;
				int rollingHashOutput = check2DRollingHash(window);
				if(rollingHashOutput == 1){ //if window content satisfies the strong condition
					//New boundary found
					fileFinished = false;
					chunk = filedata.substr(startpoint_chunk, min_chunk_size + buffer);
					isBoundaryFound = true;
					break;
				} else if(rollingHashOutput == 2 && possibleBoundary == 0){ //if window content satisfies the soft condition
					possibleBoundary = min_chunk_size + buffer;
					startpoint_window++;
					buffer++;
				} else{ //if window content does not satisty any condition
					startpoint_window++;
					buffer++;
				}
			} //Maximum chunk size reached

			if(!isBoundaryFound){
				//If the boundary has not been found, but there is a possible boundary, the chunk will cut in the possible boundary
				if(possibleBoundary!=0){
					number_of_2d_blocks++;
					chunk = filedata.substr(startpoint_chunk, possibleBoundary);
				}
				//If the boundary has not been found, force a chunk with the maximum chunk size
				else{
					number_of_forced_blocks++;
					chunk = filedata.substr(startpoint_chunk, new_max_chunk_size);
				}
			}
			//update variables for the next loop
			isBoundaryFound = false;
			possibleBoundary = 0;
			buffer = 0;
			startpoint_chunk = startpoint_chunk + chunk.size();
		}

		number_of_chunks++;

		if(keygen_mode == "conv"){
			// CONVENTIONAL ENCRYPTION
			convEncrypt(root, chunk, filename, number_of_chunks-1); //position = number_of_chunks-1
		} else{
			// MESSAGE LOCKED ENCRYPTION
			MLE(root, chunk, filename, number_of_chunks-1);//position = number_of_chunks-1
		}
	}

	/*************************** SEND TAGS AND CIPHERTEXT ****************************/

	double time_taken = (double)(clock() - tStart)/CLOCKS_PER_SEC;

	Json::FastWriter fastWriter;
	string json = fastWriter.write(root);
	cURLpost(json, STORE_FILE_ENDPOINT);

	//CDC ALGORITHM ANAlYSIS LOGS
	cout << "Time taken: " << time_taken << " ||" << endl;
	cout << "Number of chunks: " << number_of_chunks << " ||" << endl;
	cout << "Chunk size average: " << total_size/number_of_chunks << " ||" << endl;
	cout << "Number of times hash function " << number_of_times << " ||" << endl;
	cout << "Number of forced chunks " << number_of_forced_blocks << " ||" << endl;
	cout << "Number of 2nd divisor chunks " << number_of_2d_blocks << " ||" << endl;
}

void leapBasedMode(string filename, size_t total_size, size_t min_chunk_size, size_t max_chunk_size, size_t window_size){

	Json::Value root;

	ifstream file(filename, ifstream::binary);
	vector<char> data(total_size);
	file.read(&data[0], total_size); //Get the file content

	//Starting point to meauser the execution time
		clock_t tStart = clock();

	string filedata = string(data.begin(),data.end());

	bool fileFinished = false;
	int number_of_windows = 10;
	int startpoint_chunk = 0;
	size_t new_max_chunk_size = max_chunk_size;
	int number_of_chunks = 0;
	int number_of_times = 0;
	int number_of_forced_blocks = 0;

	while(!fileFinished){
		string chunk;
		//Detect if the chunk is the last one and its size is less than minimum chunk size
		if(startpoint_chunk + min_chunk_size >total_size){
			fileFinished = true;
			//Add padding to complete the chunk
			chunk = filedata.substr(startpoint_chunk, total_size - startpoint_chunk);
			//Fill the last chunk with 1+0s to get the same size in all the chunks
			/**chunk.append(1, '1');
			size_t number_of_zeros = min_chunk_size - (total_size -startpoint_chunk) - 1;
			chunk.append(number_of_zeros, '0');**/ //Uncomment these lines to enable padding
		}
		else{
			//Detect if the chunk is the last one and its size is between minimum and maximum (we have to set a new maximum threshold)
			if(startpoint_chunk + max_chunk_size > total_size){
				fileFinished = true;
				new_max_chunk_size = total_size - startpoint_chunk;
			}

			//-----------LEAP-BASED CDC ALGORITHM (1 CONDITION) -----------
			string window_set;
			int startpoint_window_set = min_chunk_size - window_size;
			bool isBoundaryFound = false;
			while(startpoint_window_set + window_size + number_of_windows < new_max_chunk_size){ //Is between min and max

				window_set = filedata.substr(startpoint_chunk + startpoint_window_set, window_size + number_of_windows - 1);
				int startpoint_window = 0;
				while(startpoint_window < number_of_windows){
					string window = window_set.substr(startpoint_window, window_size);
					number_of_times++;
					if(checkLeapBasedHash(window)){ //if window content satisfies the condition
						//Continue to the next window in the set
						startpoint_window++;
						isBoundaryFound = true;
					} else{
						//Move to the next window set
						startpoint_window_set = startpoint_window_set + startpoint_window;
						isBoundaryFound = false;
						break;
					}
				}
				if(isBoundaryFound){ //All the windows in a set have satisfy the condition
					fileFinished = false;
					chunk = filedata.substr(startpoint_chunk, startpoint_window_set + window_size + number_of_windows);
					break;
				}

			} //Maximum chunk size reached


			//If the boundary has not been found, force a chunk with the maximum chunk size
			if(!isBoundaryFound){
				number_of_forced_blocks++;
				chunk = filedata.substr(startpoint_chunk, new_max_chunk_size);
			}
			//update variables for the next loop
			isBoundaryFound = false;
			startpoint_chunk = startpoint_chunk + chunk.size();
		}

		number_of_chunks++;

		if(keygen_mode == "conv"){
			// CONVENTIONAL ENCRYPTION
			convEncrypt(root, chunk, filename, number_of_chunks-1); //position = number_of_chunks-1
		} else{
			// MESSAGE LOCKED ENCRYPTION
			MLE(root, chunk, filename, number_of_chunks-1);//position = number_of_chunks-1
		}
	}

	/*************************** SEND TAGS AND CIPHERTEXT ****************************/

	double time_taken = (double)(clock() - tStart)/CLOCKS_PER_SEC;

	Json::FastWriter fastWriter;
	string json = fastWriter.write(root);
	cURLpost(json, STORE_FILE_ENDPOINT);


	//CDC ALGORITHM ANAlYSIS LOGS
	cout << "Time taken: " << time_taken << " ||" << endl;
	cout << "Number of chunks: " << number_of_chunks << " ||" <<  endl;
	cout << "Chunk size average: " << total_size/number_of_chunks << " ||" << endl;
	cout << "Number of forced chunks " << number_of_forced_blocks << " ||" << endl;
	cout << "Number of times hash function " << number_of_times << " ||" << endl;

}

/**
 * MAIN FUNCTION
 */

int main(int argc, char* argv[])
{

	/*********************** ARGUMENTS MANAGEMENT *******************************/

  /* CASE: Different number of arguments than expected. Help message */
  if (argc < 3 || argc > 7)
    {
      printHelpMessage();
      return 1;
    }

  // 1 ARGUMENT --- Check if first argument is actually a existing file
  ifstream file(argv[1], ifstream::binary);
  if (not file)
    {
      cerr << "file: " << argv[1] << " failed to open" << endl;
      printHelpMessage();
      return 1;
    }

  /* *NIX way to get file size without seeking to the end and back */
  struct stat filestatus;
  stat(argv[1], &filestatus);

  size_t total_size = filestatus.st_size;
  cout << "total size: " << total_size << endl;

  //2 ARGUMENT --- Check keyGen algorithm
  //If it is a file, "conv" mode is selected with a stored key
  ifstream keyfile(argv[2], ifstream::binary);
  if (not keyfile){
	  //Check the KeyGen modes
	  keygen_mode = argv[2];
      if(keygen_mode != "mle" && keygen_mode != "conv"){
        	cerr << "2nd paramater not valid" << endl;
        	printHelpMessage();
        	return 1;
      }
  } else{
	  keygen_mode = "conv";
	  personal_key.assign( (std::istreambuf_iterator<char>(keyfile) ),
              (std::istreambuf_iterator<char>()    ) );
  }


  //3 ARGUMENT --- Check the chunking algorithm selected
    chunking_mode = argv[3];

  //FIXED-BLOCK DEDUPLICATION
  if(chunking_mode == "fixed"){
	    size_t chunk_size = 0;

	    /* C-string necessitates aoti to get chunk size */
	    if (argc == 5)
	      { chunk_size = atoi(argv[4]) * 1000;}

	    /* atoi may fail and leave us with an undefined chunk size*/
	    if (not (chunk_size > 0))
	      { chunk_size = 8 * 1000; }
	    cout << "using chunk size: " << chunk_size << endl;

	    //call to the algorithm
	    fixedBlocksMode(argv[1], total_size, chunk_size);

  }
  else if(chunking_mode == "bsw" || chunking_mode == "tttd" || chunking_mode == "lb"){
	  //Initialization
	  size_t min_chunk_size = 0;
	  size_t max_chunk_size = 0;
	  size_t window_size = 0;

	  //Set user values
	  if (argc >= 5){ min_chunk_size = atoi(argv[4]) * 1000;}
	  if (argc >= 6){ max_chunk_size = atoi(argv[5]) * 1000;}
	  if (argc == 7){ window_size = atoi(argv[6]);}

	  //Set default values (undefined input)
	  if (not (min_chunk_size > 0)){ min_chunk_size = 4 * 1000; }
	  if (not (max_chunk_size > 0)){ max_chunk_size = 12 * 1000; }
	  if (not (window_size > 0)){ window_size = 64; }

	  //Logs
	  cout << "using min chunk size: " << min_chunk_size << endl;
	  cout << "using max chunk size: " << max_chunk_size << endl;
	  cout << "using window size: " << window_size << endl;

	  //call to the algorithm
	  if(chunking_mode == "bsw"){
		  bswMode(argv[1], total_size, min_chunk_size, max_chunk_size, window_size);
	  } else if(chunking_mode == "tttd"){
		  tttdMode(argv[1], total_size, min_chunk_size, max_chunk_size, window_size);
	  } else {
		  leapBasedMode(argv[1], total_size, min_chunk_size, max_chunk_size, window_size);
	  }

  }else{
	  printHelpMessage();
	  return 1;
  }

}
